/* Rock.h provides the declaration of class Rock.
 *
 * Class Invariant:
 *    The value of myRockName is one of the following:
 *       BASALT, DOLOMITE, GRANITE, LIMESTONE, MARBLE, 
 *       OBSIDIAN, QUARTZITE, SANDSTONE, SHALE, SLATE
 ***************************************************************/

#ifndef ROCK
#define ROCK

#include <iostream.h>                                // istream, ostream

#include "RockKind.h"                                // class RockKind
#include "RockTexture.h"                             // class RockTexture

enum RockName {ROCK_UNDERFLOW = -1,
               BASALT, DOLOMITE, GRANITE,           // recognized
               LIMESTONE, MARBLE, OBSIDIAN,         //  names of
               QUARTZITE, SANDSTONE, SHALE, SLATE,  //  rocks
               NUMBER_OF_ROCKS,
               ROCK_OVERFLOW = NUMBER_OF_ROCKS};

class Rock
{
 public:                                             // constructors:
   Rock();                                           //  default value
   Rock(RockName initialRock);                       //  explicit value

   void Read(istream & in);                          // input
   void Print(ostream & out) const;                  // output

   RockKind Kind() const;                            // igneous, ...
   RockTexture Texture() const;                      // coarse, ...

   Rock operator++();                                // prefix ++
   Rock operator++(int);                             // postfix ++
   Rock operator--();                                // prefix --
   Rock operator--(int);                             // postfix --

                                                     // relationals
   friend bool operator==(const Rock & left, const Rock & right);  
   friend bool operator!=(const Rock & left, const Rock & right);  
   friend bool operator<(const Rock & left, const Rock & right);  
   friend bool operator>(const Rock & left, const Rock & right);  
   friend bool operator<=(const Rock & left, const Rock & right);  
   friend bool operator>=(const Rock & left, const Rock & right);  

 private:
   RockName myRockName;
 };

// -------- Non-member input ---------------------------------------
inline istream & operator>>(istream & in, Rock & theRock)
{
   theRock.Read(in);
   return in;
 }

// -------- Non-member output --------------------------------------
inline ostream & operator<<(ostream & out, const Rock & theRock)
{
   theRock.Print(out);
   return out;
 }

// -------- Initialize me (default value) --------------------------
inline Rock::Rock()
{
   myRockName = BASALT;
 }

// -------- Initialize me (explicit-value) -------------------------
#include <cassert>
inline Rock::Rock(RockName initialRock)
{
   assert(BASALT <= initialRock && initialRock < NUMBER_OF_ROCKS);
   myRockName = initialRock;
 }

//--------- Compare me and anotherRock using equality --------------
inline bool operator==(const Rock & left, const Rock & right)
{
   return left.myRockName == right.myRockName;
 }

// ---  Inline definitions for operators !=, <, >, <=, and >= are
// ---  essentially the same and are omitted here to save space.

#endif
